Get2BoundedBuffer subclass: #BodyGet2BoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Body-Examples'!


!BodyGet2BoundedBuffer methodsFor: 'state predicates'!

isOne
	^contents size = 1! !

!BodyGet2BoundedBuffer methodsFor: 'default classes'!

activityClass
	"Specification of the body."

	^Get2BoundedBufferBodyActivity! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

BodyGet2BoundedBuffer class
	instanceVariableNames: ''!


!BodyGet2BoundedBuffer class methodsFor: 'example'!

exampleGet2Size: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleGet2Size: 3 numberItems: 8 speedRatio: 2"

	super exampleGet2Size: maxSize numberItems: numberItems speedRatio: ratio! !

GgetBoundedBuffer subclass: #BodyGgetBoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Body-Examples'!


!BodyGgetBoundedBuffer methodsFor: 'default classes'!

activityClass
	"Specification of the body."

	^GgetBoundedBufferBodyActivity! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

BodyGgetBoundedBuffer class
	instanceVariableNames: ''!


!BodyGgetBoundedBuffer class methodsFor: 'example'!

exampleGgetSize: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleGgetSize: 3 numberItems: 8 speedRatio: 1"

	super exampleGgetSize: maxSize numberItems: numberItems speedRatio: ratio! !

IRBoundedBuffer subclass: #BodyBoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Body-Examples'!
BodyBoundedBuffer comment:
'Class BodyAcceptBoundedBuffer implements the bounded buffer example.
The body specifies which messages it may accept according to its state.
Note that synchronization code is outside of the method bodie, which makes it more modular.
However it is centralized within the body. As a result this model is not very good at reusing specifications because it has to fully redefine the body.
See discussion of inheritance anomalies in examples of categories Actalk-Synchro-*.'!


!BodyBoundedBuffer methodsFor: 'default classes'!

activityClass
	"Specification of the body."

	^BoundedBufferBodyActivity! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

BodyBoundedBuffer class
	instanceVariableNames: ''!


!BodyBoundedBuffer class methodsFor: 'example'!

exampleSize: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleSize: 3 numberItems: 8 speedRatio: 4"
	"self exampleSize: 3 numberItems: 8 speedRatio: 1/4"

	super exampleSize: maxSize numberItems: numberItems speedRatio: ratio! !

BodyBoundedBuffer subclass: #ObjectBodyBoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Body-Examples'!


!ObjectBodyBoundedBuffer methodsFor: 'body'!

body
	"Body: change/dispatch acceptance of messages according to the state of the buffer."

	[true] whileTrue:
		[self answer:
			(self isEmpty
				ifTrue: [#(put:)]
				ifFalse: [self isFull
							ifTrue: [#(get)]
							ifFalse: [#(get put:)]])]! !

!ObjectBodyBoundedBuffer methodsFor: 'default classes'!

activityClass
	"Delegate the body to the active object."

	^ObjectBodyActivity! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ObjectBodyBoundedBuffer class
	instanceVariableNames: ''!


!ObjectBodyBoundedBuffer class methodsFor: 'example'!

exampleSize: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleSize: 3 numberItems: 8 speedRatio: 4"
	"self exampleSize: 3 numberItems: 8 speedRatio: 1/4"

	super exampleSize: maxSize numberItems: numberItems speedRatio: ratio! !

SimulatedBodyObject subclass: #SimulatedBodyBoundedBuffer
	instanceVariableNames: 'contents maxSize '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Body-Examples'!
SimulatedBodyBoundedBuffer comment:
'Class SimulatedBodyAcceptBoundedBuffer implements the bounded buffer example.
It is equivalent to class BodyAcceptBoundedBuffer except in that it simulates the body through some initial message (see class SimulatedBodyAcceptActorBehavior).'!


!SimulatedBodyBoundedBuffer methodsFor: 'initialize'!

initialize: anInteger
	maxSize := anInteger.
	contents := BoundedBufferObject new maxSize: maxSize! !

!SimulatedBodyBoundedBuffer methodsFor: 'script'!

bodyMessage
	"Body: change/dispatch acceptance of messages according to the state of the buffer."

	[true] whileTrue:
		[self answer:
			(self isEmpty
				ifTrue: [#(put:)]
				ifFalse: [self isFull
							ifTrue: [#(get)]
							ifFalse: [#(get put:)]])]!

get
	^contents removeFirst!

put: item
	contents addLast: item! !

!SimulatedBodyBoundedBuffer methodsFor: 'state predicates'!

isEmpty
	^contents isEmpty!

isFull
	^contents isFull! !

!SimulatedBodyBoundedBuffer methodsFor: 'printing'!

printOn: aStream
	"Print as its class name, then its contents."

	super printOn: aStream.
	contents printOn: aStream! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SimulatedBodyBoundedBuffer class
	instanceVariableNames: ''!


!SimulatedBodyBoundedBuffer class methodsFor: 'instance creation'!

new: size
	^self new initialize: size! !

!SimulatedBodyBoundedBuffer class methodsFor: 'example'!

exampleSize: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleSize: 3 numberItems: 8 speedRatio: 4"
	"self exampleSize: 3 numberItems: 8 speedRatio: 1/4"

	| buffer producer consumer |
	buffer := (self new: maxSize) active.
	producer := (Producer new buffer: buffer delay: 1) active.
	consumer := (Consumer new buffer: buffer delay: ratio) active.
	producer runPut: numberItems.
	consumer runGet: numberItems! !

ExplicitAcceptActivity subclass: #BoundedBufferBodyActivity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Body-Examples'!


!BoundedBufferBodyActivity methodsFor: 'body'!

body
	"Body: change/dispatch acceptance of messages according to the state of the buffer."

	[true] whileTrue:
		[self answer:
			(oself isEmpty
				ifTrue: [#(put:)]
				ifFalse: [oself isFull
							ifTrue: [#(get)]
							ifFalse: [#(get put:)]])]! !

BoundedBufferBodyActivity subclass: #Get2BoundedBufferBodyActivity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Body-Examples'!


!Get2BoundedBufferBodyActivity methodsFor: 'body'!

body
	"Body: change/dispatch acceptance of messages according to the state of the buffer."

	[true] whileTrue:
		[self answer:
			(oself isEmpty
				ifTrue: [#(put:)]
				ifFalse: [oself isOne
							ifTrue: [#(get put:)]
							ifFalse: [oself isFull
										ifTrue: [#(get get2)]
										ifFalse: [#(get get2 put:)]]])]! !

BoundedBufferBodyActivity subclass: #GgetBoundedBufferBodyActivity
	instanceVariableNames: 'afterPut '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Body-Examples'!


!GgetBoundedBufferBodyActivity methodsFor: 'body'!

body
	"Body: change/dispatch acceptance of messages according to the state of the buffer."

	[true] whileTrue:
		[self answer:
			(oself isEmpty
				ifTrue: [#(put:)]
				ifFalse: [oself isFull
							ifTrue: [#(get)]
							ifFalse: [afterPut
										ifTrue: [#(get put:)]
										ifFalse: [#(get gget put:)]]])]! !

!GgetBoundedBufferBodyActivity methodsFor: 'events'!

kernelCompleteEvent: aMessage
	super kernelCompleteEvent: aMessage.
	afterPut := aMessage selector = #put:! !
