Address subclass: #GenericSendAddress
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-GenericSend'!


!GenericSendAddress methodsFor: 'message passing'!

futureSend: aMessage withFutureClass: futureClass inMessageQueue: queue
	"Implement future type message passing as an asynchronous message send
	with the future object inserted within the message.
	Immediately return the future object."

	| aFuture |
	aFuture := futureClass new.
	self asynchronousSend:
			(self insertReplyDestination: aFuture intoMessage: aMessage)
		inMessageQueue: queue.
	^aFuture!

insertReplyDestination: r intoMessage: aMessage
	"Insert a reply destination into a message.
	Return the extended message."
	"This is a generic virtual method. It should be defined in subclasses."

	^self subclassResponsability!

receiveMessage: aMessage
	"Generic dispatch on the message send type and mode."

	^self receiveMessage: aMessage
		withSendType: (self sendTypeOf: aMessage)
		withSendMode: (self sendModeOf: aMessage)!

receiveMessage: aMessage withSendType: type inMessageQueue: queue
	"Generic dispatch on the message send type."
	"Default recognizes only asynchronous type."

	type = #asynchronous
		ifTrue: [^self asynchronousSend: aMessage inMessageQueue: queue].
	^self error: 'unrecognized message type' , type printString!

receiveMessage: aMessage withSendType: type withSendMode: mode
	"Generic dispatch on the message send mode."
	"Default recognizes only standard mode."

	mode = #standard
		ifTrue: [^self receiveMessage: aMessage withSendType: type inMessageQueue: mailBox].
	self error: 'unrecognized message mode' , mode printString!

sendModeOf: aMessage
	"Check and return the message send mode."
	"Default is standard mode."

	^#standard!

sendTypeOf: aMessage
	"Check and return the message send type."
	"Default is asynchronous type."

	^#asynchronous!

synchronousSend: aMessage inMessageQueue: queue
	"Implement synchronous message passing as waiting onto a future type message passing."

	^(self futureSend: aMessage withFutureClass: SAFuture inMessageQueue: queue) value! !
