CountersActivity subclass: #ASCActivity
	instanceVariableNames: 'enabledSelectors '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-ASC'!
ASCActivity comment:
'Class ASCActorBehavior implements a mixed synchronization model.
ASC stands for Abstract State and Synchronization Counters.
This means using both state transitions to specify state synchronization and activation conditions (guards based on synchronization counters) to specify activation synchronization.
As a result specification is more modular and reusable. However these two synchronization dimensions are not fully orthogonal. (See example of classes ASCGetRearBoundedBuffer and ASCGetRear2BoundedBuffer).
This model has been proposed by Laurent Thomas at PARLE92 Conference.
Class ASCActorBehavior is a combination of synchronization classes StateActorBehavior and CountersActorBehavior. However because there is no multiple inheritance, we choose the most complex class, CountersActorBehavior as the superclass, and we recopy the state transition part part. Meanwhile we need to be careful about the combination, because we have to ensure atomicity of checking of conditions and method completion. (This is actually easily specified through simple refinement of method checking the guard to also include the enabling check.)'!


!ASCActivity methodsFor: 'initialize'!

privateInitialize

	super privateInitialize.		"Create the three counters dictionaries."
	"Compute and assign the initial collection of enabled selectors."
	enabledSelectors := self perform: self initialAbstractState! !

!ASCActivity methodsFor: 'synchro events'!

synchroEventComplete: aMessage
	super synchroEventComplete: aMessage.
	enabledSelectors := self perform: (self nextAbstractStateAfter: aMessage selector)! !

!ASCActivity methodsFor: 'guard testing'!

satisfyGuardSelector: selector arguments: argumentsArray
	"Check if the message belongs to current set of enabled selectors
	AND if the guard evaluates to true.
	Both checks must be done within the same atomic block."

	^(enabledSelectors includes: selector)
		and: [super satisfyGuardSelector: selector arguments: argumentsArray]! !

!ASCActivity methodsFor: 'abstract states'!

initialAbstractState
	"Return the initial abstract state.
	Mandatory. Must be supplied by the class or one of its superclass.
	Must return some abstract state symbol/selector."

	^self subclassResponsability! !

!ASCActivity methodsFor: 'state transition'!

nextAbstractStateAfter: selector
	"Specify and compute next abstract state.
	Must return some abstract state symbol/selector.
	Default is by delegating this specification/computation to the active object."

	^oself nextAbstractStateAfter: selector! !
