GuardsActivity subclass: #TableGuardsActivity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-Guards-Ex'!


!TableGuardsActivity methodsFor: 'guards'!

guardOFpickUp: i and: j
	"Both forks must be available."

	^oself areForksAvailableAt: i and: j!

guardOFputDown: i and: j
	"Always free to release forks."

	^true!

guardOFstart
	^true! !

GgetBoundedBuffer subclass: #GuardsGgetBoundedBuffer
	instanceVariableNames: 'afterPut '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-Guards-Ex'!


!GuardsGgetBoundedBuffer methodsFor: 'default classes'!

activityClass
	"The guards associated specification."

	^GgetBoundedBufferGuardsActivity! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

GuardsGgetBoundedBuffer class
	instanceVariableNames: ''!


!GuardsGgetBoundedBuffer class methodsFor: 'example'!

exampleGgetSize: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleGgetSize: 3 numberItems: 8 speedRatio: 1"

	super exampleGgetSize: maxSize numberItems: numberItems speedRatio: ratio! !

ImplicitReplyObject subclass: #GuardsPhilosopher
	instanceVariableNames: 'leftForkPosition rightForkPosition table numberSteps '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-Guards-Ex'!
GuardsPhilosopher comment:
'Class GuardsPhilosopher implements the dining philosophers example with guards.
As opposed to the solution with POOL explicit message acceptance (see class PoolPhilosopher) which needed to restrict the number of philosophers accessing the table in order to esure deadlock avoidance, this description does not need to.
The solution makes use of a class GuardsTable to centralize access to the forks in order to ensure that a philosopher picks up his two forks at once (atomically) if they are both available.
However this slightly restricts the ideal concurrency of accesses to forks, that is it serializes accesses to forks even if two philosophers use different forks.'!


!GuardsPhilosopher methodsFor: 'initialize'!

position: anInteger n: n table: aTable numberSteps: steps
	leftForkPosition := anInteger.
	rightForkPosition := (leftForkPosition \\ n) + 1.
	table := aTable.
	numberSteps := steps! !

!GuardsPhilosopher methodsFor: 'script'!

start
	"Activity of a philosopher."

	numberSteps timesRepeat:
		[self think.
		Transcript show: self printString , ' wants to eat and requests forks '
			, leftForkPosition printString , ' and ' , rightForkPosition printString; cr.
		"He requests his forks. Note that he waits (value) for the acknowledgement."
		(table pickUp: leftForkPosition and: rightForkPosition) value.
		self eat.
		"Note that when releasing forks, he does not need to wait for acknowledgement to proceed."
		table putDown: leftForkPosition and: rightForkPosition.
		Transcript show: self printString , ' has finished eating and releases forks '
			, leftForkPosition printString , ' and ' , rightForkPosition printString; cr]! !

!GuardsPhilosopher methodsFor: 'private routines'!

eat
	| delay |
	delay := (Random new next * 5) rounded + 1.
	Transcript show: self printString , ' eats during ' , delay printString , ' seconds.'; cr.
	(Delay forSeconds: delay) wait!

think
	| delay |
	delay := (Random new next * 5) rounded + 1.
	Transcript show: self printString , ' thinks during ' , delay printString , ' seconds.'; cr.
	(Delay forSeconds: delay) wait! !

!GuardsPhilosopher methodsFor: 'printing'!

printOn: aStream
	aStream nextPutAll: self class name , leftForkPosition printString! !

Get2BoundedBuffer subclass: #GuardsGet2BoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-Guards-Ex'!


!GuardsGet2BoundedBuffer methodsFor: 'state predicates'!

hasAtLeastTwoElements
	^contents size >= 2! !

!GuardsGet2BoundedBuffer methodsFor: 'default classes'!

activityClass
	"The guards associated specification."

	^Get2BoundedBufferGuardsActivity! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

GuardsGet2BoundedBuffer class
	instanceVariableNames: ''!


!GuardsGet2BoundedBuffer class methodsFor: 'example'!

exampleGet2Size: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleGet2Size: 3 numberItems: 8 speedRatio: 2"

	super exampleGet2Size: maxSize numberItems: numberItems speedRatio: ratio! !

GuardsObject subclass: #GuardsTable
	instanceVariableNames: 'n numberSteps philosophers forksAvailability '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-Guards-Ex'!
GuardsTable comment:
'Class GuardsTable implements the shared ressource of philosophers, that is forks placed onto the table. See class GuardsPhilosopher comment.'!


!GuardsTable methodsFor: 'initialize'!

initialize: anInteger numberSteps: steps
	n := anInteger.
	numberSteps := steps!

makeForksAvailability
	"Create and initialize the array containing the availability of each fork."

	forksAvailability := Array new: n withAll: true!

makePhilosophers
	"Create and initialize the n philosophers."

	philosophers := Array new: n.
	1 to: n do:
		[:i | philosophers at: i put:
			(GuardsPhilosopher new
				position: i
				n: n
				table: aself
				numberSteps: numberSteps)
					active]! !

!GuardsTable methodsFor: 'script'!

pickUp: i and: j
	forksAvailability at: i put: false;
				at: j put: false!

putDown: i and: j
	forksAvailability at: i put: true;
				at: j put: true!

start
	self makeForksAvailability.
	self makePhilosophers.
	philosophers do:
		[:philosopher | philosopher start]! !

!GuardsTable methodsFor: 'state predicates'!

areForksAvailableAt: i and: j
	^(forksAvailability at: i) & (forksAvailability at: j)! !

!GuardsTable methodsFor: 'default classes'!

activityClass
	^TableGuardsActivity! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

GuardsTable class
	instanceVariableNames: ''!


!GuardsTable class methodsFor: 'example'!

example: n
	"GuardsTable example: 5"

	(GuardsTable new initialize: n numberSteps: 3) actor start! !

GuardsActivity subclass: #BoundedBufferGuardsActivity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-Guards-Ex'!


!BoundedBufferGuardsActivity methodsFor: 'guards'!

guardOFget
	"Not empty."

	^oself isEmpty not!

guardOFput: item
	"Not full."

	^oself isFull not! !

BoundedBufferGuardsActivity subclass: #GgetBoundedBufferGuardsActivity
	instanceVariableNames: 'afterPut '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-Guards-Ex'!


!GgetBoundedBufferGuardsActivity methodsFor: 'events'!

kernelEventComplete: aMessage
	afterPut := aMessage selector = #put:! !

!GgetBoundedBufferGuardsActivity methodsFor: 'guards'!

guardOFgget
	"Not empty and not after a put."

	^(self guardOFget) & (afterPut not)! !

BoundedBufferGuardsActivity subclass: #Get2BoundedBufferGuardsActivity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-Guards-Ex'!


!Get2BoundedBufferGuardsActivity methodsFor: 'guards'!

guardOFget2
	"At least two elements."

	^oself hasAtLeastTwoElements! !

IRBoundedBuffer subclass: #Guards1BoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-Guards-Ex'!


!Guards1BoundedBuffer methodsFor: 'default classes'!

activityClass
	"The guards associated specification."

	^BoundedBufferGuards1Activity! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Guards1BoundedBuffer class
	instanceVariableNames: ''!


!Guards1BoundedBuffer class methodsFor: 'example'!

exampleSize: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleSize: 3 numberItems: 8 speedRatio: 4"
	"self exampleSize: 3 numberItems: 8 speedRatio: 1/4"

	super exampleSize: maxSize numberItems: numberItems speedRatio: ratio! !

Guards1BoundedBuffer subclass: #GuardsBoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-Guards-Ex'!


!GuardsBoundedBuffer methodsFor: 'default classes'!

activityClass
	"The guards associated specification."

	^BoundedBufferGuardsActivity! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

GuardsBoundedBuffer class
	instanceVariableNames: ''!


!GuardsBoundedBuffer class methodsFor: 'example'!

exampleSize: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleSize: 3 numberItems: 8 speedRatio: 4"
	"self exampleSize: 3 numberItems: 8 speedRatio: 1/4"

	super exampleSize: maxSize numberItems: numberItems speedRatio: ratio! !

Guards1Activity subclass: #BoundedBufferGuards1Activity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-Guards-Ex'!


!BoundedBufferGuards1Activity methodsFor: 'guards'!

guardOFget
	"Not empty."

	^oself isEmpty not!

guardOFput: item
	"Not full."

	^oself isFull not! !
